package unibo.cs.labss.editor;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import unibo.cs.labss.editor.model.IEntity;
import unibo.cs.labss.editor.model.IModel;
import unibo.cs.labss.editor.model.Model;

/**
 * @author Riccardo Solmi
 */
public abstract class AbstractExamplesFactory implements IExamplesFactory {
	private Map registry = new HashMap(); // String to Factory

  	public IEntity createEmptyModel() {
		return create("empty");
	}

	public IEntity create(String type) {
		ExampleFactory factoryMethod = (ExampleFactory)registry.get(type);
		
		if (factoryMethod == null)
			throw new IllegalArgumentException("esempio non registrato: "+type);
		return setModel(factoryMethod.create());
	}

	public void put(String type, ExampleFactory factoryMethod) {
		registry.put(type, factoryMethod);
	}
	public void remove(String type) {
		registry.remove(type);
	}
	public Set keySet() {
		return registry.keySet();
	}

	private IEntity setModel(IEntity root) {
		IModel model = createModel(getModelId());
		model.setRoot(root);
		root.setModel(model);
		return root;
	}
    protected IModel createModel(String modelId) {
    	return new Model(modelId);
    }
    protected abstract String getModelId();
}
