package unibo.cs.labss.editor.actions;

import java.util.List;

import org.eclipse.ui.IWorkbenchPart;

import unibo.cs.labss.editor.IPrototypeManager;
import unibo.cs.labss.editor.commands.CompositeAddCommand;
import unibo.cs.labss.editor.parts.AbstractPart;
import unibo.cs.labss.editor.policies.IHighlightable;
import unibo.cs.labss.editor.util.UniqueID;

/*
 * 
 * @author Riccardo Solmi
 */
public class CompositeAddAction extends AbstractLanguageAction {
	private IEnablerPredicate enablerPredicate; 
	private IPrototypeManager prototypeManager;
	private String type;
	private int lastPosition;
	
	public CompositeAddAction(IWorkbenchPart part, IPrototypeManager prototypeManager, IEnablerPredicate enablerPredicate, String type, String text) {
		super(part);
		this.enablerPredicate = enablerPredicate;
		this.prototypeManager = prototypeManager;
		this.type = type;

		if (!prototypeManager.containsKey(type))
			throw new IllegalArgumentException(type+" is not a prototype");

		setImageDescriptor(INSERT_ICON);
//		setId("add "+prototypeManager.toString()+"."+type);
		setId(UniqueID.getUniqueID());
		setText(text);
	}

	public boolean isLanguageActionEnabled() {
		boolean enabled = false;

		List selectedObjects = getSelectedObjects();
		if (selectedObjects.size() == 1 && selectedObjects.get(0) instanceof AbstractPart) {
			enabled = enablerPredicate.evaluate(
					(AbstractPart)selectedObjects.get(0),
					new ContextMenuRequest(prototypeManager, type));
		}

		if (enabled) {
			AbstractPart selectedPart = (AbstractPart)getSelectedObjects().get(0);
			lastPosition = ((IHighlightable)selectedPart).getHilightPosition();
			return true;
		} else {
			lastPosition = -1;
			return false;
		}
	}

	protected boolean calculateEnabled() {
		return lastPosition >= 0;
	}

	public void run() {
		AbstractPart selectedPart = (AbstractPart)getSelectedObjects().get(0);
		
		CompositeAddCommand add = new CompositeAddCommand();
		add.setComposite(selectedPart.getModelEntity());
		add.setNewChild(prototypeManager.clone(type));
		add.setIndex(lastPosition);
		execute(add);

		lastPosition = -1;
	}
}
