package unibo.cs.labss.editor.actions;

import java.util.Iterator;

import unibo.cs.labss.editor.parts.AbstractPart;
import unibo.cs.labss.editor.policies.IHighlightable;
import unibo.cs.labss.editor.requests.PartRequest;

/**
 * @author Riccardo Solmi
 */
public class EnablerPredicateFactory {
	public final IEnablerPredicate and(final IEnablerPredicate p1, final IEnablerPredicate p2) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return p1.evaluate(selectedPart, userdata) && p2.evaluate(selectedPart, userdata);
			}
		};
	}
	public final IEnablerPredicate and(final IEnablerPredicate p1, final IEnablerPredicate p2, final IEnablerPredicate p3) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return p1.evaluate(selectedPart, userdata) && p2.evaluate(selectedPart, userdata) && p3.evaluate(selectedPart, userdata);
			}
		};
	}
	public final IEnablerPredicate or(final IEnablerPredicate p1, final IEnablerPredicate p2) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return p1.evaluate(selectedPart, userdata) || p2.evaluate(selectedPart, userdata);
			}
		};
	}
	public final IEnablerPredicate or(final IEnablerPredicate p1, final IEnablerPredicate p2, final IEnablerPredicate p3) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return p1.evaluate(selectedPart, userdata) || p2.evaluate(selectedPart, userdata) || p3.evaluate(selectedPart, userdata);
			}
		};
	}
	public final IEnablerPredicate or(final IEnablerPredicate p1, final IEnablerPredicate p2, final IEnablerPredicate p3, final IEnablerPredicate p4) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return p1.evaluate(selectedPart, userdata) || p2.evaluate(selectedPart, userdata) || p3.evaluate(selectedPart, userdata) || p4.evaluate(selectedPart, userdata);
			}
		};
	}
	public final IEnablerPredicate not(final IEnablerPredicate p) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return !p.evaluate(selectedPart, userdata);
			}
		};
	}

	public final IEnablerPredicate instanceOf(final Class entityType) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return entityType.isInstance(selectedPart.getModel());
			}
		};
	}
	public final IEnablerPredicate parent_instanceOf(final Class entityType) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return entityType.isInstance(selectedPart.getParentModelEntity());
			}
		};
	}

	public final IEnablerPredicate sameType() {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				ContextMenuRequest request = (ContextMenuRequest)userdata;
				return request.isInstance(selectedPart.getModelEntity());
			}
		};
	}
	public final IEnablerPredicate differentType() {
		return not(sameType());
	}
	
	public final IEnablerPredicate insertMode() {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return selectedPart instanceof IHighlightable &&
				((IHighlightable)selectedPart).getHilightPosition() >= 0;
			}
		};
	}
	public final IEnablerPredicate replaceMode() {
		return not(insertMode());
	}

	// DnD Enabler Predicates
	public final IEnablerPredicate hasCompositeParent() {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return selectedPart.getParentModelEntity().isComposite();
			}
		};
	}

	public final IEnablerPredicate isComposite() {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return selectedPart.getModelEntity().isComposite();
			}
		};
	}

	public final IEnablerPredicate dndSinglePart() {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return ((PartRequest)userdata).size() == 1;
			}
		};
	}

	public final IEnablerPredicate dndPartsInstanceOf(final Class entityType) {
		return new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				Iterator i = ((PartRequest)userdata).iterator();
				while (i.hasNext()) {
					AbstractPart part = (AbstractPart)i.next();
					if (!entityType.isInstance(part.getModelEntity()))
						return false;
				}
				return true;
			}
		};
	}

	public final IEnablerPredicate dndOver(final Class dndClass, final Class overClass) {
		return and(dndPartsInstanceOf(dndClass), instanceOf(overClass));
	}
}
