package unibo.cs.labss.editor.figures;

import java.util.List;

import org.eclipse.draw2d.AbstractLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * @author Riccardo Solmi
 */
public abstract class AbstractMathLayout extends AbstractLayout {
	protected IFigure[] figure;
	protected Dimension[] prefSize;
	protected int[] ascent;
	protected int[] descent;
	protected int[] x;
	protected int[] y;
	protected int figWidth;
	protected int figAscent;
	protected int figDescent;

	protected Dimension calculatePreferredSize(IFigure container, int wHint, int hHint) {
		List children = container.getChildren();

		figure = new IFigure[children.size()];
		prefSize = new Dimension[children.size()];
		ascent = new int[children.size()];
		descent = new int[children.size()];
		x = new int[children.size()];
		y = new int[children.size()];

		for (int i=0; i<children.size(); i++) {
			figure[i] = (IFigure)children.get(i);
			prefSize[i] = figure[i].getPreferredSize();
			if (figure[i] instanceof IBaselinedFigure) {
				ascent[i] = ((IBaselinedFigure)figure[i]).getAscent();
				descent[i] = ((IBaselinedFigure)figure[i]).getDescent();
			} else {
				ascent[i] = (int)Math.floor(prefSize[i].height/2);
				descent[i] = (int)Math.ceil(prefSize[i].height/2);
			}
		}
		
		setAscentDescentWidth();

		Rectangle result = new Rectangle().setLocation(container.getClientArea().getLocation());
		result.width = figWidth;
		result.height = figAscent + figDescent;
		result.resize(container.getInsets().getWidth(), container.getInsets().getHeight());
		return result.getSize();
	}
	protected abstract void setAscentDescentWidth();

	public void layout(IFigure container) {
		Rectangle area = container.getClientArea();
		Rectangle bounds = Rectangle.SINGLETON;

		setLocation(area);
		
		for (int i=0; i<figure.length; i++) {
			bounds.setLocation(x[i], y[i]);
			bounds.setSize(prefSize[i].width, prefSize[i].height);
			figure[i].setBounds(bounds);			
		}
	}
	protected abstract void setLocation(Rectangle area);

	public int getAscent(IFigure container) {
		if (preferredSize == null)
			preferredSize = calculatePreferredSize(container, -1, -1);
		return figAscent;
	}
	public int getDescent(IFigure container) {
		if (preferredSize == null)
			preferredSize = calculatePreferredSize(container, -1, -1);
		return figDescent;
	}
}
