package unibo.cs.labss.editor.figures;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.AbstractLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * @author Riccardo Solmi, Luca Padovani
 */
public class BoxListLayout extends AbstractLayout {
	static class Constraint {
		IBaselinedFigure figure;
		boolean isJolly;
		
		public Constraint(IBaselinedFigure figure, boolean isJolly) {
			this.figure = figure;
			this.isJolly = isJolly;
		}
	}
	static class Row {
		List constraintList = new ArrayList();
		int ascent;
		int descent;

		public boolean add(Constraint o) {
			return constraintList.add(o);
		}
		public Iterator iterator() {
			return constraintList.iterator();
		}
	}
	protected List/*<Row<Constraint>>*/ rowList;
	private int row = 0;
	protected int hgap = 4;
	protected int vgap = 4;

	public BoxListLayout() {
		rowList = new ArrayList();
		rowList.add(new Row());
	}

	public void setHorizontalSpacing(int horizontalSpacing) {
		this.hgap = horizontalSpacing;
	}
	public void setVerticalSpacing(int verticalSpacing) {
		this.vgap = verticalSpacing;
	}

	public synchronized void next() {
		if (rowList.size() <= ++row)
			rowList.add(new Row());
	}
	public synchronized void addJollyConstraint() {
		addConstraint(new Constraint(null, true));
	}
	public void addConstraint(IBaselinedFigure child) {
		invalidate(child);
		addConstraint(new Constraint(child, false));
	}
	protected synchronized void addConstraint(Constraint constraint) {
		((Row)rowList.get(row)).add(constraint);
	}
	
	protected synchronized Dimension calculatePreferredSize(IFigure container, int wHint, int hHint) {
//deleteme		container.validate();
		Rectangle result = new Rectangle().setLocation(container.getClientArea().getLocation());

		List children = container.getChildren();
		int index = 0;
		boolean isFirstRow = true;
		Iterator i = rowList.iterator();
		while (i.hasNext()) {
			Row row = (Row)i.next();
			boolean isFirstCol = true;
			int rowWidth = 0;
			int rowAscent = 0;
			int rowDescent = 0;

			Iterator j = row.iterator();
			while (j.hasNext()) {
				Constraint c = (Constraint)j.next();
			
				if (c.isJolly)
					c.figure = (IBaselinedFigure)children.get(index++);
				else if (!c.figure.isVisible())
					continue;
				
				if (isFirstCol)
					isFirstCol = false;
				else
					rowWidth += hgap;
				rowWidth += c.figure.getPreferredSize().width;
				rowAscent = Math.max(rowAscent, c.figure.getAscent());
				rowDescent = Math.max(rowDescent, c.figure.getDescent());
			}
			row.ascent = rowAscent;
			row.descent = rowDescent;
			int rowHeight = rowAscent + rowDescent;
			result.width = Math.max(result.width, rowWidth);
			if (rowHeight > 0) {
				if (isFirstRow)
					isFirstRow = false;
				else
					rowHeight += vgap;
				result.height += rowHeight;
			}
		}
		result.resize(container.getInsets().getWidth(), container.getInsets().getHeight());
		return result.getSize();
	}

	public synchronized void layout(IFigure container) {
		Rectangle clientArea = container.getClientArea();
		Rectangle bounds = Rectangle.SINGLETON;

		List children = container.getChildren();
		int index = 0;

		int y = clientArea.y;
		Iterator i = rowList.iterator();
		while (i.hasNext()) {
			Row row = (Row)i.next();
			int x = clientArea.x;
			int ymax = 0;
			
			Iterator j = row.iterator();
			while (j.hasNext()) {
				Constraint c = (Constraint)j.next();

				if (c.isJolly)
					c.figure = (IBaselinedFigure)children.get(index++); //TODO perch serve??
				else 
					if (!c.figure.isVisible())
					continue;
				
				Dimension d = c.figure.getPreferredSize();
				bounds.setLocation(x, y + row.ascent-c.figure.getAscent());
				bounds.setSize(d.width, d.height);
				c.figure.setBounds(bounds);

				x += d.width + hgap;
			}
			ymax = row.ascent+row.descent;
			if (ymax > 0)
				y += ymax + vgap;
		}
	}
}
