package unibo.cs.labss.editor.figures;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;

/**
 * is a decorator for BoxListLayout
 * @author Riccardo Solmi
 */
public class BoxListSwitchableLayout extends BoxListLayout {
	private BoxListLayout[] layout;
	private Set[] figureSet;
	private int index = 0;

	public BoxListSwitchableLayout(int size) {
		layout = new BoxListLayout[size];
		figureSet = new Set[size];
		for (int i=0; i<size; i++) {
			layout[i] = new BoxListLayout();
			figureSet[i] = new HashSet();
		}
	}

	public synchronized void selectLayout(int index) {
		if (this.index == index)
			return;

		invalidate();
		
		Iterator i = figureSet[this.index].iterator();
		while (i.hasNext())
			((IFigure)i.next()).setVisible(false);
		
		this.index = index;

		i = figureSet[index].iterator();
		while (i.hasNext())
			((IFigure)i.next()).setVisible(true);
	}
	public int layoutIndex() {
		return index;
	}
	
	protected void addConstraint(Constraint constraint) {
		layout[index].addConstraint(constraint);
	}
	public void addConstraint(IBaselinedFigure child) {
		figureSet[index].add(child);
		layout[index].addConstraint(child);
	}
	public void addJollyConstraint() {
		layout[index].addJollyConstraint();
	}
	public boolean equals(Object obj) {
		return layout[index].equals(obj);
	}
	public Object getConstraint(IFigure child) {
		return layout[index].getConstraint(child);
	}
	public Dimension getMinimumSize(IFigure container, int wHint, int hHint) {
		return layout[index].getMinimumSize(container, wHint, hHint);
	}
	public Dimension getPreferredSize(IFigure container, int wHint, int hHint) {
		return layout[index].getPreferredSize(container, wHint, hHint);
	}
	public int hashCode() {
		return layout[index].hashCode();
	}
	public void invalidate() {
		layout[index].invalidate();
	}
	public void layout(IFigure container) {
		layout[index].layout(container);
	}
	public void next() {
		layout[index].next();
	}
	public void remove(IFigure child) {
		layout[index].remove(child);
	}
	public void setConstraint(IFigure child, Object constraint) {
		layout[index].setConstraint(child, constraint);
	}
	public void setHorizontalSpacing(int horizontalSpacing) {
		layout[index].setHorizontalSpacing(horizontalSpacing);
	}
	public void setVerticalSpacing(int verticalSpacing) {
		layout[index].setVerticalSpacing(verticalSpacing);
	}
	public String toString() {
		return layout[index].toString();
	}

	protected Dimension calculatePreferredSize(IFigure container, int wHint, int hHint) {
		return layout[index].calculatePreferredSize(container, wHint, hHint);
	}
}
