package unibo.cs.labss.editor.figures;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.ui.IEditorPart;

import unibo.cs.labss.LabssPlugin;
import unibo.cs.labss.PreferenceConstants;

/**
 * @author Riccardo Solmi
 */
public class FigurePrefs extends PreferenceConstants {
	public static Color selectionColor;
	public static Color matchingSelectionColor;
	public static Color hostLanguageColor;
	public static Color templateLanguageColor;

	public static Color defaultColor = new Color(null, 255, 255, 255);
	public static Color keywordsColor;
	public static Font keywordsFont;
	public static Color operatorsColor;
	public static Font operatorsFont;
	public static Color delimitersColor;
	public static Font delimitersFont;
	public static Color parenthesisColor;
	public static Font parenthesisFont;
	public static Color literalsColor;
	public static Font literalsFont;
	public static Color identifiersColor;
	public static Font identifiersFont;
	public static Color emptyColor;
	public static Font emptyFont;


	public static void initializeValues(final IPreferenceStore store) {
		store.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				final String prop = event.getProperty();

				if (prop.equals(FONT)) {
					keywordsFont = updateFont(KEYWORDS_COLOR, keywordsFont);
					operatorsFont = updateFont(OPERATORS_COLOR, operatorsFont);
					delimitersFont = updateFont(DELIMITERS_COLOR, delimitersFont);
					parenthesisFont = updateFont(PARENTHESIS_COLOR, parenthesisFont);
					literalsFont = updateFont(LITERALS_COLOR, literalsFont);
					identifiersFont = updateFont(IDENTIFIERS_COLOR, identifiersFont);
					emptyFont = updateFont(EMPTY_COLOR, emptyFont);

				} else if (prop.equals(SELECTION_COLOR))
					selectionColor = updateColor(SELECTION_COLOR, selectionColor);
				else if (prop.equals(MATCHING_SELECTION_COLOR))
					matchingSelectionColor = updateColor(MATCHING_SELECTION_COLOR, matchingSelectionColor);
				else if (prop.equals(HOST_LANGUAGE_COLOR))
					hostLanguageColor = updateColor(HOST_LANGUAGE_COLOR, hostLanguageColor);
				else if (prop.equals(TEMPLATE_LANGUAGE_COLOR))
					templateLanguageColor = updateColor(TEMPLATE_LANGUAGE_COLOR, templateLanguageColor);

				else if (prop.equals(KEYWORDS_COLOR))
					keywordsColor = updateColor(KEYWORDS_COLOR, keywordsColor);
				else if (prop.startsWith(KEYWORDS_COLOR))
					keywordsFont = updateFont(KEYWORDS_COLOR, keywordsFont);

				else if (prop.equals(OPERATORS_COLOR))
					operatorsColor = updateColor(OPERATORS_COLOR, operatorsColor);
				else if (prop.startsWith(OPERATORS_COLOR))
					operatorsFont = updateFont(OPERATORS_COLOR, operatorsFont);

				else if (prop.equals(DELIMITERS_COLOR))
					delimitersColor = updateColor(DELIMITERS_COLOR, delimitersColor);
				else if (prop.startsWith(DELIMITERS_COLOR))
					delimitersFont = updateFont(DELIMITERS_COLOR, delimitersFont);

				else if (prop.equals(PARENTHESIS_COLOR))
					parenthesisColor = updateColor(PARENTHESIS_COLOR, parenthesisColor);
				else if (prop.startsWith(PARENTHESIS_COLOR))
					parenthesisFont = updateFont(PARENTHESIS_COLOR, parenthesisFont);

				else if (prop.equals(LITERALS_COLOR))
					literalsColor = updateColor(LITERALS_COLOR, literalsColor);
				else if (prop.startsWith(LITERALS_COLOR))
					literalsFont = updateFont(LITERALS_COLOR, literalsFont);

				else if (prop.equals(IDENTIFIERS_COLOR))
					identifiersColor = updateColor(IDENTIFIERS_COLOR, identifiersColor);
				else if (prop.startsWith(IDENTIFIERS_COLOR))
					identifiersFont = updateFont(IDENTIFIERS_COLOR, identifiersFont);

				else if (prop.equals(EMPTY_COLOR))
					emptyColor = updateColor(EMPTY_COLOR, emptyColor);
				else if (prop.startsWith(EMPTY_COLOR))
					emptyFont = updateFont(EMPTY_COLOR, emptyFont);

				invalidateEditors();
			}

			private final Color updateColor(String key, Color oldColor) {
				Color newColor = new Color(null, PreferenceConverter.getColor(store, key));
				oldColor.dispose();
				return newColor;
			}

			private final Font updateFont(String key, Font oldFont) {
				Font newFont = deriveFont(PreferenceConverter.getFontDataArray(store, FONT), getStyle(key));
				oldFont.dispose();
				return newFont;
			}
			private final int getStyle(String key) {
				int style = SWT.NORMAL;
				if (store.getBoolean(key+BOLD))
					style += SWT.BOLD;
				if (store.getBoolean(key+ITALIC))
					style += SWT.ITALIC;
				
				return style;
			}
		});

		selectionColor = getColor(store, SELECTION_COLOR);
		matchingSelectionColor = getColor(store, MATCHING_SELECTION_COLOR);
		hostLanguageColor = getColor(store, HOST_LANGUAGE_COLOR);
		templateLanguageColor = getColor(store, TEMPLATE_LANGUAGE_COLOR);

		keywordsColor = getColor(store, KEYWORDS_COLOR);
		keywordsFont = getFont(store, KEYWORDS_COLOR);
		operatorsColor = getColor(store, OPERATORS_COLOR);
		operatorsFont = getFont(store, OPERATORS_COLOR);
		delimitersColor = getColor(store, DELIMITERS_COLOR);
		delimitersFont = getFont(store, DELIMITERS_COLOR);
		parenthesisColor = getColor(store, PARENTHESIS_COLOR);
		parenthesisFont = getFont(store, PARENTHESIS_COLOR);
		literalsColor = getColor(store, LITERALS_COLOR);
		literalsFont = getFont(store, LITERALS_COLOR);
		identifiersColor = getColor(store, IDENTIFIERS_COLOR);
		identifiersFont = getFont(store, IDENTIFIERS_COLOR);
		emptyColor = getColor(store, EMPTY_COLOR);
		emptyFont = getFont(store, EMPTY_COLOR);
		
		invalidateEditors();
	}

	private static final Color getColor(IPreferenceStore store, String key) {
		return new Color(null, PreferenceConverter.getColor(store, key));
	}

	private static final Font getFont(IPreferenceStore store, String key) {
		int style = SWT.NORMAL;
		if (store.getBoolean(key+BOLD))
			style += SWT.BOLD;
		if (store.getBoolean(key+ITALIC))
			style += SWT.ITALIC;

		return deriveFont(PreferenceConverter.getFontDataArray(store, FONT), style);
	}
	private static final Font deriveFont(FontData[] fd, int style) {
		for (int i=0; i<fd.length; i++) 
			fd[i].setStyle(style);
		return new Font(null, fd);
	}
	
	private static final void invalidateEditors() {
		IEditorPart[] editors = LabssPlugin.getGraphicalEditors();
		for (int i=0; i<editors.length; i++) {
			IFigure rootFigure = ((GraphicalEditPart)((GraphicalViewer)editors[i].getAdapter(GraphicalViewer.class)).getContents()).getFigure();
			rootFigure.invalidateTree(); //TODO asynch?
			rootFigure.validate();
			rootFigure.repaint();
		}

	}
}
