package unibo.cs.labss.editor.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

public class StyledLabel extends BaselinedLabel {
	public static StyledLabel createKeyword(String name) {
		return new StyledLabel(name) {
			public Color getLocalForegroundColor() {
				return FigurePrefs.keywordsColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.keywordsFont;
			}
		};
	}
	public static StyledLabel createOperator(String name) {
		return new StyledLabel(name) {
			public Color getLocalForegroundColor() {
				return FigurePrefs.operatorsColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.operatorsFont;
			}
		};
	}
	public static StyledLabel createDelimiter(String name) {
		return new StyledLabel(name) {
			public Color getLocalForegroundColor() {
				return FigurePrefs.delimitersColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.delimitersFont;
			}
		};
	}
	public static StyledLabel createParenthesis(String name) {
		return new StyledLabel(name) {
			public Color getLocalForegroundColor() {
				return FigurePrefs.parenthesisColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.parenthesisFont;
			}
		};
	}
	public static StyledLabel createLiteral() {
		return new SelectableStyledLabel() {
			public Color getLocalForegroundColor() {
				return FigurePrefs.literalsColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.literalsFont;
			}
		};
	}
	public static StyledLabel createIdentifier() {
		return new SelectableStyledLabel() {
			public Color getLocalForegroundColor() {
				return FigurePrefs.identifiersColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.identifiersFont;
			}
		};
	}
	public static StyledLabel createEmpty() {
		return new SelectableStyledLabel() {
			{
				setText("?");
			}
			public Color getLocalForegroundColor() {
				return FigurePrefs.emptyColor;
			}	
			public Font getLocalFont() {
				return FigurePrefs.emptyFont;
			}
		};
	}
	
	public StyledLabel() {
	}
	public StyledLabel(String text) {
		setText(text);
	}

	public void paint(Graphics graphics) {
		if (getLocalBackgroundColor() != null)
			graphics.setBackgroundColor(getLocalBackgroundColor());
		if (getLocalForegroundColor() != null)
			graphics.setForegroundColor(getLocalForegroundColor());
		if (getLocalFont() != null)
			graphics.setFont(getLocalFont());

		graphics.pushState();
		try {
			paintFigure(graphics);
			graphics.restoreState();
			paintClientArea(graphics);
			paintBorder(graphics);
		} finally {
			graphics.popState();
		}
	}
	
	public Font getFont() {
		if (getLocalFont() != null)
			return getLocalFont();
		if (getParent() != null)
			return getParent().getFont();
		return null;
	}
	public Font getLocalFont() {
		return null;
	}
}
