package unibo.cs.labss.editor.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * @author Riccardo Solmi
 */
public abstract class AbstractCompositeEntity extends AbstractEntity {
	protected ArrayList children;

	public AbstractCompositeEntity() {
		this(new IEntity[0]);
	}
	public AbstractCompositeEntity(IEntity[] entities) {
		children = new ArrayList(entities.length);
		for (int i = 0; i < entities.length; i++)
			children.add(entities[i]);
	}

    protected void setChildrenModel(IModel model) {
    	Iterator i = children.iterator();
    	while (i.hasNext())
    		((IEntity)i.next()).setModel(model);
    }

    public Object clone() {
    	AbstractCompositeEntity bs = (AbstractCompositeEntity)super.clone();
		bs.children = (ArrayList)children.clone();
		return bs;
	}

	public boolean isComposite() {
		return true;
	}

	public int size() {
		return children.size();
	}
	public int indexOf(IEntity child) {
		return children.indexOf(child);
	}
	public Iterator iterator() {
		return children.iterator();
	}
	public ListIterator listIterator() {
		return children.listIterator();
	}
	public List getChildren() {
		return (List)children.clone();
	}

/*
	public void setChilden(List children) {
		this.children = new ArrayList(children);
		// setChildModel(for each new child);
		fireStructureChange();
	}

	public void addChild(IEntity child) {
		children.add(child);
		setChildModel(child);
		fireStructureChange();
	}
*/

	public void addChild(int index, IEntity child) {
		children.add(index, child);
		notifyChanged(null, child);
	}
	public void removeChild(IEntity child, IEntity emptyReplacement) {
		if (emptyReplacement != null && children.size() == 1)
			replaceChild(child, emptyReplacement);
		else
			removeChild(child);
	}
	public void removeChild(IEntity child) {
		children.remove(child);
		notifyChanged(child, null);
	}


	public void replaceChild(IEntity oldChild, IEntity newChild) {
		int index = children.indexOf(oldChild);
		if (index >= 0)
			notifyChanged((IEntity)children.set(index, newChild), newChild);
	}
}
