package unibo.cs.labss.editor.model;

import java.io.Serializable;
import java.util.List;

import unibo.cs.labss.LabssMessages;


/**
 * @author Riccardo Solmi
 */
public abstract class AbstractEntity implements IEntity, Serializable, Cloneable {
    private IModel model = NullModel.instance;

    public IModel getModel() {
    	return model;
    }
    public final void setModel(IModel model) {
    	if (this.model != model)
    		setChildrenModel(this.model = model);
    }
    protected void setChildrenModel(IModel model) {
    }
    private final void setChildModel(IEntity child) {
    	child.setModel(getModel());
    }

    public String getModelId() {
    	return getModel().getModelId();
    }
    public IEntity getRoot() {
    	return getModel().getRoot();
    }

    public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			throw new InternalError();
		}
	}

    protected void notifyChanged(IEntity oldValue, IEntity newValue) {
    	notifyChanged("CHILDREN", oldValue, newValue);
    }
    protected void notifyChanged(String propertyName, IEntity oldValue, IEntity newValue) {
    	if (newValue != null)
    		setChildModel(newValue);

   		getModel().notifyChanged(this, propertyName, oldValue, newValue);
    }
    protected void notifyChanged(String propertyName, boolean oldValue, boolean newValue) {
   		getModel().notifyChanged(this, propertyName, oldValue, newValue);
    }
    protected void notifyChanged(String propertyName, int oldValue, int newValue) {
   		getModel().notifyChanged(this, propertyName, oldValue, newValue);
    }
    protected void notifyChanged(String propertyName, Object oldValue, Object newValue) {
   		getModel().notifyChanged(this, propertyName, oldValue, newValue);
    }

	public boolean isComposite() {
		return false;
	}

	public int size() {
		throw new IllegalArgumentException(LabssMessages.no_composite);
	}
	public int indexOf(IEntity child) {
		throw new IllegalArgumentException(LabssMessages.no_composite);
	}	
	public List getChildren() {
		throw new IllegalArgumentException(LabssMessages.no_composite);
	}
	public void addChild(int index, IEntity child) {
		throw new IllegalArgumentException(LabssMessages.no_composite);
	}
	public void removeChild(IEntity child, IEntity emptyReplacement) {
		throw new IllegalArgumentException(LabssMessages.no_composite);
	}
	public void replaceChild(IEntity oldChild, IEntity newChild) {
		throw new IllegalArgumentException(LabssMessages.no_composite);
	}
}
