package unibo.cs.labss.editor.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.gef.AccessibleEditPart;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.tools.SelectEditPartTracker;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.widgets.Display;

import unibo.cs.labss.editor.model.IEntity;
import unibo.cs.labss.editor.policies.AltDragEditPartsTracker;
import unibo.cs.labss.editor.policies.LanguageContainerEditPolicy;
import unibo.cs.labss.editor.policies.LanguageComponentEditPolicy;
import unibo.cs.labss.editor.policies.LanguageHighlightEditPolicy;
import unibo.cs.labss.editor.policies.LanguageLayoutEditPolicy;
import unibo.cs.labss.editor.requests.ICommandFactory;

/*
 * @author Riccardo Solmi
 */
public abstract class AbstractPart extends AbstractGraphicalEditPart implements PropertyChangeListener {
    private AccessibleEditPart acc;
	private boolean draggable;
	private ICommandFactory commandFactory;
	
	public void setCommandFactory(ICommandFactory commandFactory) {
		this.commandFactory = commandFactory;
	}
	public ICommandFactory getCommandFactory() {
		return commandFactory;
	}
	
	public AbstractPart() {
		this(true);
	}
	public AbstractPart(boolean isDraggable) {
		setDraggable(isDraggable);
	}

    protected void createEditPolicies() {
    	installEditPolicy(EditPolicy.CONTAINER_ROLE, new LanguageContainerEditPolicy(getCommandFactory()));
    	installEditPolicy(EditPolicy.COMPONENT_ROLE, new LanguageComponentEditPolicy(getCommandFactory()));
    	installEditPolicy(EditPolicy.LAYOUT_ROLE, new LanguageLayoutEditPolicy(getCommandFactory()));
    	installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, new LanguageHighlightEditPolicy());
    }
    public DragTracker getDragTracker(Request request) {
    	return isDraggable() ? new AltDragEditPartsTracker(this) : new SelectEditPartTracker(this);
	}

	public boolean isDraggable() {
		return draggable;
	}
	public void setDraggable(boolean draggable) {
		this.draggable = draggable;
	}
    
	public IEntity getModelEntity() {
		return (IEntity)getModel();
	}
	public IEntity getParentModelEntity() {
		Object parent = getParent().getModel();
		return parent instanceof IEntity ? (IEntity)parent : null;
	}

	/**
     * Handles changes in properties of this. It is 
     * activated through the PropertyChangeListener.
     * It updates children, source and target connections,
     * and the visuals of this based on the property
     * changed.
     *
     * @param evt  Event which details the property change.
     */
    public final void propertyChange(final PropertyChangeEvent evt){
    	Display.getDefault().asyncExec(new Runnable() {
    		public void run() {
    			propertyChangeUI(evt);
    		}
    	});
    }
	protected void propertyChangeUI(PropertyChangeEvent evt) {
			refreshChildren();
		// Causes Graph to re-layout	
		//((GraphicalEditPart)(getViewer().getContents())).getFigure().revalidate();
	}

	protected AccessibleEditPart createAccessible() {
	    return new AccessibleGraphicalEditPart(){
	        public void getName(AccessibleEvent e) {
	            e.result = getModel().toString();
	        }
	    };
	}
	protected AccessibleEditPart getAccessibleEditPart() {
	    if (acc == null)
	        acc = createAccessible();
	    return acc;
	}

	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT || request.getType() == RequestConstants.REQ_OPEN)
			performDirectEdit();
	}

	protected void performDirectEdit() {
	}
}
