package unibo.cs.labss.editor.parts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.gef.requests.SelectionRequest;
import org.eclipse.gef.tools.DeselectAllTracker;
import org.eclipse.gef.tools.MarqueeDragTracker;


/*
 * Main EditPart responsible for the background figure of the view
 * Concrete subclasses must override the getModelChildren method
 *  
 * @author Riccardo Solmi
 */
public class AbstractRootPart extends AbstractPart implements LayerConstants {
	public AbstractRootPart() {
		super(false);
	}

	protected void createEditPolicies(){
		super.createEditPolicies();
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());
    	installEditPolicy(EditPolicy.LAYOUT_ROLE, null);
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, null);
		installEditPolicy(EditPolicy.NODE_ROLE, null);
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
	}
	
	public DragTracker getDragTracker(Request req){
		if (req instanceof SelectionRequest && ((SelectionRequest)req).getLastButtonPressed() == 3)
			return new DeselectAllTracker(this);
		return new MarqueeDragTracker();
	}

	protected IFigure createFigure() {
	    Figure f = new FreeformLayer();
	    ToolbarLayout l = new ToolbarLayout();
	    l.setSpacing(32);
	    l.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
	    l.setStretchMinorAxis(false);
	    f.setLayoutManager(l);
	    f.setBorder(new MarginBorder(5));
	    f.setBackgroundColor(ColorConstants.listBackground);

	    f.setOpaque(true);
	    return f;
	}
}
