package unibo.cs.labss.editor.parts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import unibo.cs.labss.LabssPlugin;
import unibo.cs.labss.editor.IEditorKit;
import unibo.cs.labss.editor.IPartFactory;
import unibo.cs.labss.editor.model.IEntity;
import unibo.cs.labss.editor.requests.ICommandFactory;
import unibo.cs.labss.editors.EditorKitExtensions;

/**
 * @author Riccardo Solmi
 */
public class ModelEditPartFactory implements EditPartFactory {
	// last used model is cached
	private String modelId;
	private IPartFactory partFactory;
	private ICommandFactory commandFactory;
	
	public EditPart createEditPart(EditPart context, Object entity) {
		try {
			IEntity modelEntity = (IEntity)entity;
			String modelId = modelEntity.getModelId();
			if (!modelId.equals(this.modelId)) {
				this.modelId = modelId;
				IEditorKit editorKit = EditorKitExtensions.instance().getEditorKit(modelId);
				partFactory = editorKit.getPartFactory();
				commandFactory = editorKit.getCommandFactory();
			}

			AbstractPart part = partFactory.createPart(modelEntity);
	        part.setModel(modelEntity);
	        part.setCommandFactory(commandFactory);
	        return part;
		} catch (Exception e) {
			LabssPlugin.log(e);
			throw new IllegalArgumentException("Illegal model entity: "+entity+" in context: "+context);
		}
	}	
}
